/*
Copirigth 2013,SIANI - ULPGC
Jose Juan Hernandez Cabrera
Jose Evora Gomez
Johan Sebastian Cortes Montenegro

This File is Part of JavaFMI Library

JavaFMI Library is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

JavaFMI Library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.
*/

package org.javafmi;

import java.util.ArrayList;
import java.util.List;
import org.simpleframework.xml.Attribute;
import org.simpleframework.xml.Element;
import org.simpleframework.xml.ElementList;
import org.simpleframework.xml.Root;

@Root(name = "fmiModelDescription")
public class ModelDescription {

    public static final String REAL_TYPE_ID = "real";
    public static final String INTEGER_TYPE_ID = "integer";
    public static final String BOOLEAN_TYPE_ID = "boolean";
    public static final String STRING_TYPE_ID = "string";
    @Attribute(required = true)
    private String fmiVersion;
    @Attribute(required = true)
    private String modelName;
    @Attribute(required = false)
    private String modelIdentifier;
    @Attribute(required = true)
    private String guid;
    @Attribute(required = false)
    private String generationTool;
    @Attribute(required = false)
    private String generationDateAndTime;
    @Attribute(required = false)
    private String variableNamingConvention;
    @Attribute(required = false)
    private int numberOfContinuousStates;
    @Attribute(required = false)
    private int numberOfEventIndicators;
    @Element(name = "ModelVariables", required = false)
    private ModelVariables modelVariables;
    @Element(name = "Implementation", required = false)
    private Implementation implementation;
    @ElementList(name = "TypeDefinitions", required = false)
    private List<DymolaType> typeDefinitions;
    @Element(name = "DefaultExperiment", required = false)
    private DefaultExperiment defaultExperiment;
    @Element(name = "CoSimulation", required = false)
    private CoSimulation coSimulation;
    @Element(name = "ModelStructure", required = false)
    private ModelStructure modelStructure;

    public String getFmiVersion() {
        return fmiVersion;
    }

    public String getModelName() {
        return modelName;
    }

    public String getModelIdentifier() {
        return modelIdentifier;
    }

    public String getGuid() {
        return guid;
    }

    public int getNumberOfContinuousStates() {
        return numberOfContinuousStates;
    }

    public int getNumberOfEventIndicators() {
        return numberOfEventIndicators;
    }

    public ModelVariables getModelVariables() {
        return modelVariables;
    }

    public Implementation getImplementation() {
        return implementation;
    }

    public boolean isCosimulation() {
        return (implementation != null);
    }

    @Root
    public static class TypeDefinitions {

        @ElementList(name = "Type", required = false, inline = true)
        private DymolaType type;
    }

    @Root
    public static class DymolaType {

        @Attribute(name = "name", required = false)
        private String name;
        @Element(name = "RealType", required = false)
        private RealType realType;
        @Element(name = "Real", required = false)
        private RealType real;
    }

    public static class DefaultExperiment {

        @Attribute(required = false)
        private double startTime;
        @Attribute(required = false)
        private double stopTime;
        @Attribute(required = false)
        private double tolerance;
    }

    public static class ModelVariables {

        @ElementList(name = "ScalarVariable", inline = true)
        ArrayList<ScalarVariable> scalarVariableList;

        public ArrayList<ScalarVariable> getScalarVariableList() {
            return scalarVariableList;
        }

        public ScalarVariable getScalarVariable(String name) {
            for (ScalarVariable scalarVariable : scalarVariableList)
                if (scalarVariable.getName().equals(name))
                    return scalarVariable;
            return null;
        }
    }

    @Root(name = "ScalarVariable")
    public static class ScalarVariable {

        @Attribute(required = true)
        private String name;
        @Attribute(required = false)
        private int valueReference;
        @Attribute(required = false)
        private String description;
        @Attribute(required = false)
        private String variability;
        @Attribute(required = false)
        private String alias;
        @Attribute(required = false)
        private String causality;
        @Element(name = "Real", required = false)
        private RealType realType;
        @Element(name = "Integer", required = false)
        private IntegerType integerType;
        @Element(name = "Boolean", required = false)
        private BooleanType booleanType;
        @Element(name = "String", required = false)
        private StringType stringType;
        @Element(name = "DirectDependency", required = false)
        private DirecDependency direcDependency;

        public String getName() {
            return name;
        }

        public Integer getValueReference() {
            return valueReference;
        }

        public String getDescription() {
            return description;
        }

        public String getVariability() {
            return variability;
        }

        public String getAlias() {
            return alias;
        }

        public String getCasuality() {
            return causality;
        }

        public Type getType() {
            if (realType != null)
                return realType;
            if (integerType != null)
                return integerType;
            if (booleanType != null)
                return booleanType;
            if (stringType != null)
                return stringType;
            return null;
        }
    }

    public static class DirecDependency {
    }

    public static abstract class Type {

        @Attribute(required = false)
        private boolean fixed;

        public boolean isFixed() {
            return fixed;
        }

        public abstract String getType();
    }

    @Root(name = "Real")
    public static class RealType extends Type {

        @Attribute(required = false)
        private String declaredType;
        @Attribute(required = false)
        private double start;

        public double getStart() {
            return start;
        }

        @Override
        public String getType() {
            return REAL_TYPE_ID;
        }
    }

    @Root(name = "Integer")
    public static class IntegerType extends Type {

        @Attribute(required = false)
        private int start;

        public int getStart() {
            return start;
        }

        @Override
        public String getType() {
            return INTEGER_TYPE_ID;
        }
    }

    @Root(name = "Boolean")
    public static class BooleanType extends Type {

        @Attribute(required = false)
        private boolean start;

        public boolean getStart() {
            return start;
        }

        @Override
        public String getType() {
            return BOOLEAN_TYPE_ID;
        }
    }

    @Root(name = "String")
    public static class StringType extends Type {

        @Attribute(required = false)
        private String start;

        public String getStart() {
            return start;
        }

        @Override
        public String getType() {
            return STRING_TYPE_ID;
        }
    }

    public static class Implementation {

        @Element(name = "CoSimulation_StandAlone", required = false)
        CoSimulation coSimulation;

        public CoSimulation getCoSimulation() {
            return coSimulation;
        }
    }

    @Root(name = "CoSimulation_StandAlone")
    public static class CoSimulation {

        @Attribute(required = false)
        private String modelIdentifier;
        @Attribute(required = false)
        private String canHandleVariableCommunicationStepSize;
        @Attribute(required = false)
        private String canHandleEvents;
        @Attribute(required = false)
        private String canInterpolateInputs;
        @Attribute(required = false)
        private String maxOutputDerivativeOrder;
        @Attribute(required = false)
        private String canBeInstantiatedOnlyOncePerProcess;
        @Attribute(required = false)
        private String canGetAndSetFMUstate;
        @Attribute(required = false)
        private String providesDirectionalDerivatives;
        @Element(name = "Capabilities", required = false)
        Capabilities capabilities;

        public Capabilities getCapabilities() {
            return capabilities;
        }
    }

    @Root(name = "Capabilities")
    public static class Capabilities {

        @Attribute(required = false)
        boolean canHandleVariableCommunicationStepSize;
        @Attribute(required = false)
        boolean canHandleEvents;
        @Attribute(required = false)
        boolean canBeInstantiatedOnlyOncePerProcess;

        public boolean isCanHandleVariableCommunicationStepSize() {
            return canHandleVariableCommunicationStepSize;
        }

        public boolean isCanHandleEvents() {
            return canHandleEvents;
        }
    }

    @Root
    public static class ModelStructure {

        @ElementList(name = "Inputs", required = false)
        private List<Input> inputs;
        @ElementList(name = "Derivatives", required = false)
        private List<Derivative> derivatives;
    }

    public static class Input {

        @Attribute(required = false)
        private String name;
    }

    public static class Derivative {

        @Attribute(required = false)
        private String name;
        @Attribute(required = false)
        private String state;
    }
}
